@@ -280,7 +280,7 @@ use strict;
 use warnings;
 use bytes;
 
-our $VERSION = '1.23';
+our $VERSION = '1.25';
 
 use Cache::FastMmap::CImpl;
 
@@ -467,6 +467,7 @@ sub new {
     $share_file = ($^O eq "MSWin32" ? "c:\\sharefile" : "/tmp/sharefile");
     $share_file .= "-" . $$ . "-" . time;
   }
+  !ref($share_file) || die "share_file argument was a reference";
   $Self->{share_file} = $share_file;
 
   my $init_file = $Args{init_file} || 0;
@@ -743,17 +744,24 @@ sub get_and_set {
   return $Value;
 }
 
-=item I<remove($Key)>
+=item I<remove($Key, [ \%Options ])>
 
 Delete the given key from the cache
 
+I<%Options> is optional, and is used by get_and_remove() to control
+the locking behaviour. For now, you should probably ignore it
+unless you read the code to understand how it works
+
 =cut
 sub remove {
   my ($Self, $Cache) = ($_[0], $_[0]->{Cache});
 
   # Hash value, lock page, read result
   my ($HashPage, $HashSlot) = $Cache->fc_hash($_[1]);
-  $Cache->fc_lock($HashPage);
+
+  # Lock is done only if we're not in the middle of a get_and_remove() operation.
+  $Cache->fc_lock($HashPage) unless $_[2] && $_[2]->{skip_lock};
+
   my ($DidDel, $Flags) = $Cache->fc_delete($HashSlot, $_[1]);
   $Cache->fc_unlock();
 
@@ -767,6 +775,23 @@ sub remove {
   return $DidDel;
 }
 
+=item I<get_and_remove($Key)>
+
+Atomically retrieve value of a Key while removing it from the cache.
+
+The page is locked while retrieving the $Key and is unlocked only after
+the value is removed, thus guaranteeing the value stored by someone else
+isn't removed by us.
+
+=cut
+sub get_and_remove {
+  my ($Self, $Cache) = ($_[0], $_[0]->{Cache});
+
+  my $Value = $Self->get($_[1], { skip_unlock => 1 });
+  my $DidDel = $Self->remove($_[1], { skip_lock => 1 });
+  return wantarray ? ($Value, $DidDel) : $Value;
+}
+
 =item I<clear()>
 
 Clear all items from the cache
@@ -905,7 +930,7 @@ sub multi_get {
   return \%KVs;
 }
 
-=item I<multi_set($PageKey, { $Key1 => $Value1, $Key2 => $Value2, ... })>
+=item I<multi_set($PageKey, { $Key1 => $Value1, $Key2 => $Value2, ... }, [ \%Options ])>
 
 Store specified key/value pair into cache
 
@@ -913,6 +938,10 @@ Store specified key/value pair into cache
 sub multi_set {
   my ($Self, $Cache) = ($_[0], $_[0]->{Cache});
 
+  # Get opts, make compatiable with Cache::Cache interface
+  my $Opts = defined($_[3]) ? (ref($_[3]) ? $_[3] : { expire_time => $_[3] }) : undef;
+  my $expire_seconds = defined($Opts && $Opts->{expire_time}) ? parse_expire_time($Opts->{expire_time}) : -1;
+
   # Hash page key value, lock page
   my ($HashPage, $HashSlot) = $Cache->fc_hash($_[1]);
   $Cache->fc_lock($HashPage);
@@ -932,7 +961,7 @@ sub multi_set {
 
     # Now hash key and store into page
     (undef, $HashSlot) = $Cache->fc_hash($FinalKey);
-    $Cache->fc_write($HashSlot, $FinalKey, $Val, 0);
+    $Cache->fc_write($HashSlot, $FinalKey, $Val, $expire_seconds, 0);
   }
 
   # Unlock page
